@@ -1,21 +1,21 @@
+#include <iostream>
+#include <random>
 #include <Eigen/Core>
 #include <Eigen/SparseCore>
-#include <iostream>
-#include <random>  // Requires C++ 11
-
 #include <Spectra/JDSymEigsBase.h>
 #include <Spectra/MatOp/DenseGenMatProd.h>
 
-using namespace Spectra;
-
 #include "catch.hpp"
 
+using namespace Spectra;
+
 using Matrix = Eigen::MatrixXd;
 using Vector = Eigen::VectorXd;
 using ComplexMatrix = Eigen::MatrixXcd;
 using ComplexVector = Eigen::VectorXcd;
 using SpMatrix = Eigen::SparseMatrix<double>;
 using Index = Eigen::Index;
+
 template <typename OpType>
 class JDMock : public JDSymEigsBase<JDMock<OpType>, OpType>
 {
@@ -40,4 +40,4 @@ TEST_CASE("Constructing JDSymObject", "[eigs_gen]")
     JDMock<DenseGenMatProd<double>> eigs(op, 5);
     REQUIRE(eigs.num_iterations() == 0);
     REQUIRE(eigs.info() == Spectra::CompInfo::NotComputed);
-}
\ No newline at end of file
+}
