@@ -1,17 +1,16 @@
-#include <Eigen/Core>
-#include <Eigen/SparseCore>
 #include <iostream>
 #include <type_traits>
-#include <random>  // Requires C++ 11
-
+#include <random>
+#include <Eigen/Core>
+#include <Eigen/SparseCore>
 #include <Spectra/SymEigsSolver.h>
 #include <Spectra/MatOp/DenseSymMatProd.h>
 #include <Spectra/MatOp/SparseSymMatProd.h>
 
-using namespace Spectra;
-
 #include "catch.hpp"
 
+using namespace Spectra;
+
 using Matrix = Eigen::MatrixXd;
 using Vector = Eigen::VectorXd;
 using SpMatrix = Eigen::SparseMatrix<double>;
